//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <vector>
#include <iostream.h>
#include <fstream>
using namespace std;
#include "SubAccount.h"
#include "Account.h"
#include "DebtMain.h"
#include "ActForm.h"
#include "PayPlan.h"
#include "AboutDlg.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    UpdateGrid();
    Application->HelpFile="DebtHelp.hlp";
}
//---------------------------------------------------------------------------

void TMainForm::UpdateGrid()
{
    DataGrid->RowCount=accounts.size()+2;
    DataGrid->Cells[0][0]="Account";
    DataGrid->Cells[1][0]="Balance";
    DataGrid->Cells[2][0]="Interest";
    DataGrid->Cells[3][0]="Minimum";

    double balance,interest,min;
    balance=interest=min=0;
    for (unsigned int i=0;i<accounts.size();i++)
    {
        DataGrid->Cells[0][i+1]=accounts[i]->GetName();
        DataGrid->Cells[1][i+1]=accounts[i]->GetBalance();
        balance+=accounts[i]->GetBalance();
        DataGrid->Cells[2][i+1]=accounts[i]->GetInterest();
        interest+=accounts[i]->GetInterest();
        DataGrid->Cells[3][i+1]=accounts[i]->GetMinPayment();
        min+=accounts[i]->GetMinPayment();
    }

    DataGrid->Cells[0][DataGrid->RowCount-1]="Total";
    DataGrid->Cells[1][DataGrid->RowCount-1]=balance;
    DataGrid->Cells[2][DataGrid->RowCount-1]=interest;
    DataGrid->Cells[3][DataGrid->RowCount-1]=min;
}

void __fastcall TMainForm::AddButtonClick(TObject *Sender)
{
    CAccount *ptr;
    ptr=AccountForm->EditAccount(NULL);
    if (ptr)
    {
        accounts.push_back(ptr);
        UpdateGrid();
    }
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::DoGridDblClick(TObject *Sender)
{
    int n=DataGrid->Row;
    if (n==0 || n==(DataGrid->RowCount-1))
        return;
    CAccount *ptr=accounts[n-1];
    ptr=AccountForm->EditAccount(ptr);
    if (ptr)
    {
        CAccount *tmp=accounts[n-1];
        accounts[n-1]=ptr;
        UpdateGrid();
        delete tmp;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PlanButtonClick(TObject *Sender)
{
    TStrings *out=PlanForm->Memo->Lines;
    out->Clear();
    CalcPlan(out);
    PlanForm->ShowModal();
}
//---------------------------------------------------------------------------


void TMainForm::CalcPlan(TStrings *out)
{
    vector<CAccount*> acts;
    double balance,min,pay,amt,paid,tmp,interest,total;
    unsigned int i;
    String str;
    balance=min=0;
    for (i=0;i<accounts.size();i++)
    {
        acts.push_back(accounts[i]->Clone());
        balance+=accounts[i]->GetBalance();
        min+=accounts[i]->GetMinPayment();
    }
    //SortAccounts(acts);

    str=min;
    str=InputBox("Payment Amount","Enter Amount",str);
    try
    {
        pay=str.ToDouble();
    }
    catch (EConvertError *e)
    {
        pay=min;
    }

    TDateTime date=date.CurrentDate();
    interest=total=0;
    while (balance>0)
    {
        SortAccounts(acts);
        out->Add("----- "+date.FormatString("mmmm yyyy")+" -----");
        paid=0;
        for(i=0;i<acts.size();i++)
        {
            if (acts[i]->GetBalance()>0)
            {
                interest+=acts[i]->GetInterest();
                min-=acts[i]->GetMinPayment();
                amt=pay-paid-min;
                tmp=acts[i]->MakePayment(amt);
                paid+=amt-tmp;

                str=GetLine(acts[i]->GetName(),amt-tmp,acts[i]->GetBalance());
                out->Add(str);
            }
        }
        total+=paid;
        out->Add("");
        unsigned short y,m,d;
        date.DecodeDate(&y,&m,&d);
        if (m<12)
            date=EncodeDate(y,m+1,d);
        else
            date=EncodeDate(y+1,1,d);
        balance=GetBalance(acts);
        min=GetMinimum(acts);
    }
    for(i=0;i<acts.size();i++)
        delete acts[i];

    out->Add("-------------- Summary --------------");
    out->Add("Total Paid:    $"+AnsiString::FormatFloat("#,###,##0.00",total));
    out->Add("Interest Paid: $"+AnsiString::FormatFloat("#,###,##0.00",interest));
}

double TMainForm::GetBalance(vector<CAccount*> &acts)
{
    double balance=0;
    for (unsigned int i=0;i<acts.size();i++)
        balance+=acts[i]->GetBalance();
    return balance;
}

double TMainForm::GetMinimum(vector<CAccount*> & acts)
{
    double min=0;
    for (unsigned int i=0;i<acts.size();i++)
        min+=acts[i]->GetMinPayment();
    return min;
}

void __fastcall TMainForm::DeleteAccountClick(TObject *Sender)
{
    int n=DataGrid->Row;
    if (n==0 || n==(DataGrid->RowCount-1))
        return;
    vector<CAccount*>::iterator iter=accounts.begin();
    iter+=n-1;
    CAccount *ptr=accounts[n-1];
    accounts.erase(iter);
    delete ptr;
    UpdateGrid();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DoCloseing(TObject *Sender,
      TCloseAction &Action)
{
    Action = caFree;
    for (unsigned int i=0;i<accounts.size();i++)
        delete accounts[i];
}
//---------------------------------------------------------------------------


void TMainForm::SortAccounts(vector<CAccount*> & acts)
{
    int i,j,size;
    double avgint1,avgint2,bal;
    CAccount* ptr;

    size=acts.size();
    for (i=0;i<size-1;i++)
        for (j=i+1;j<size;j++)
        {
            bal=acts[i]->GetBalance();
            if (bal!=0)
                avgint1=acts[i]->GetInterest()/bal;
            else
                avgint1=-1;
            bal=acts[j]->GetBalance();
            if (bal!=0)
                avgint2=acts[j]->GetInterest()/bal;
            else
                avgint2=-1;
            if (avgint1<avgint2)
            {
                ptr=acts[i];
                acts[i]=acts[j];
                acts[j]=ptr;
            }
        }
}

String TMainForm::GetLine(String name, double paid, double balance)
{
    String str,tmp;

    str=name+":";
    /*
    if (name.Length()<8)
        str+="\t";
    if (name.Length()<16)
        str+="\t";
    */
    while (str.Length()<16)
        str+="        ";

    str+=" \t";
    str+="$"+AnsiString::FormatFloat("#,###,##0.00",paid)+"\t ";
    str+="$"+AnsiString::FormatFloat("#,###,##0.00",balance);
    return str;
}

void __fastcall TMainForm::Exit1Click(TObject *Sender)
{
    Close();    
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::About1Click(TObject *Sender)
{
    TAboutBox *about=new TAboutBox(this);
    about->ShowModal();
    delete about;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Open1Click(TObject *Sender)
{
    OpenDialog->Title="Open";
    if (OpenDialog->Execute())
    {
        for (unsigned int i=0;i<accounts.size();i++)
            delete accounts[i];
        accounts.clear();

        ifstream in(OpenDialog->FileName.c_str(),ios::in | ios::binary);

        char buffer[256];
        in.read(buffer,16);
        // todo: check validity of file
        int numact,numsubs,i,j;
        CAccount *ptr;
        CSubAccount *sub;
        int tmp;

        in.read((char*)&numact,sizeof(numact));
        for (i=0;i<numact;i++)
        {
            ptr=new CAccount();
            j=0;
            do
            {
                in.read(&buffer[j],1);
            }
            while (buffer[j++]!=0);
            ptr->name=buffer;
            in.read((char*)&(ptr->MinAbs),sizeof(double));
            in.read((char*)&(ptr->MinPercent),sizeof(double));
            in.read((char*)&numsubs,sizeof(numsubs));
            for (j=0;j<numsubs;j++)
            {
                sub=new CSubAccount();
                in.read((char*)&(sub->balance),sizeof(double));
                in.read((char*)&(sub->first_rate),sizeof(double));
                in.read((char*)&(sub->second_rate),sizeof(double));
                in.read((char*)&tmp,sizeof(tmp));
                sub->change=tmp;
                sub->current=TDateTime::CurrentDate();
                ptr->subs.push_back(sub);
            }
            accounts.push_back(ptr);
        }
    }
    UpdateGrid();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Save1Click(TObject *Sender)
{
    OpenDialog->Title="Save As";
    if (OpenDialog->Execute())
    {
        String fname=OpenDialog->FileName;
        if (fname.LastDelimiter(".")==0)
            fname+=".dat";
        ofstream out(fname.c_str(),ios::out | ios::binary);
        
        out.write("Debt File v0.8  ",16);
        unsigned int tmp=accounts.size();
        out.write((char*)&tmp,sizeof(tmp));
        for (unsigned int i=0;i<accounts.size();i++)
        {
            out.write(accounts[i]->name.c_str(),accounts[i]->name.Length());
            out.write("\0",1);
            out.write((char*)&(accounts[i]->MinAbs),sizeof(double));
            out.write((char*)&(accounts[i]->MinPercent),sizeof(double));
            tmp=accounts[i]->subs.size();
            out.write((char*)&tmp,sizeof(tmp));
            for (unsigned int j=0;j<accounts[i]->subs.size();j++)
            {
                out.write((char*)&(accounts[i]->subs[j]->balance),sizeof(double));
                out.write((char*)&(accounts[i]->subs[j]->first_rate),sizeof(double));
                out.write((char*)&(accounts[i]->subs[j]->second_rate),sizeof(double));
                tmp=(int)accounts[i]->subs[j]->change;
                out.write((char*)&tmp,sizeof(tmp));
            }
        }
        out.flush();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::HelpTopics1Click(TObject *Sender)
{
    Application->HelpCommand(HELP_CONTENTS,0);    
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::New1Click(TObject *Sender)
{
    for (unsigned int i=0;i<accounts.size();i++)
        delete accounts[i];
    accounts.clear();
    UpdateGrid();
}
//---------------------------------------------------------------------------

